#!/bin/bash

#    Interpréteur lilypond vers Beep, version 0.1.0
#    Copyright (C) 2020  Marco De Freitas
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#--INITIALISATION
#-------Langue du système de notation

# Hollandais, langue par défaut en lilypond
#name1="a" ; name2="b" ; name3="c" ; name4="d" ; name5="e" ; name6="f" ; name7="g" ; rest="r" ; altup="is" ; altdown="es" ;

# Anglais,
#name1="a" ; name2="b" ; name3="c" ; name4="d" ; name5="e" ; name6="f" ; name7="g" ; rest="r" ; altup="s" ; altdown="s" ;

# Allemand
#name1="a" ; name2="h" ; name3="c" ; name4="d" ; name5="e" ; name6="f" ; name7="g" ; rest="r" ; altup="is" ; altdown="es" ;

# Français
name1="la" ; name2="si" ; name3="do" ; name4="ré" ; name5="mi" ; name6="fa" ; name7="sol" ; rest="r" ; altup="d" ; altdown="b" ;

# Italien
#name1="la" ; name2="si" ; name3="do" ; name4="re" ; name5="mi" ; name6="fa" ; name7="sol" ; rest="r" ; altup="d" ; altdown="b" ;

# -----EXPRESSION REGULIERE
regex="($name1|$name2|$name3|$name4|$name5|$name6|$name7|$rest)($altup+|$altdown+)?([,]+|[']+)?(32|16|[1248])*([\.]*)"


#--FONCTIONS

# Cette fonction prend pour paramètre le premier groupe de capture ($name1|$name2 etc...)
# et renvoie la valeur midi correspondante.

function nomVersMidi
{
local note
	case $1 in
		"$name1")
			echo "69"
			;;
		"$name2")
			echo "71"
			;;
		"$name3")
			echo "72"
			;;
		"$name4")
			echo "74"
			;;
		"$name5")
			echo "76"
			;;
		"$name6")
			echo "77"
			;;
		"$name7")
			echo "79"
			;;
		"$rest")
			echo "0"
	esac
}



# Cette fonction prends pour paramètre les altérations
# et elle renvoie la valeur à ajouter ou retrancher
# une note midi = un demi-ton
# !!!la fonction se limite à prendre en compte une seule altération,
# donc pas de doubles, triples bémols/dièses....

function alterationVersMidi
{ 
	case $1 in
		"$altup")
			echo  "+1" #monte d'un demi-ton
			;;
		"$altdown")
			echo  "-1" #descend d'un demi ton
			;;
		"*")
			echo "0"
			;;
	esac	
}


# Cette fonction prends pour paramètre les indications de registre
# elle renvoie la valeur à ajouter ou retrancher
# douze notes midi = une octave
# 
# à déterminer quels registres sont pertinents (freq <20hz inaudible)

function registreVersMidi
{
	case $1 in
		",")
			echo "-12" #descend d'une octave
			;;
		",,")
			echo "-24" #descend de deux octaves
			;;	
		"'")
			echo "+12" #monte d'une octave
			;;
		"''")
			echo "+24" #monte de deux octaves
			;;
		"*")
			echo "0"
			;;
	esac
}




# Cette fonction détermine la hauteur en note midi à partir du nom, des altérations, du registre
# elle appelle les fonctions de conversion vers midi respectives en plaçant le résultat de la capture en paramètre
# l'addition de la valeur midi du nom de l'altération et du registre donne la valeur midi de la hauteur

function hauteurVersMidi
{
nbMidi=$(nomVersMidi $1)
if [[ $1 = "0"  ]]
	then
		echo "0"
	else
		if [ -z $2 ]   #teste si une altération à été capturée
			then	
				paramAlteration="0" #si pas d'altération alors valeur 0
			else		
				paramAlteration=$(alterationVersMidi $2) #calcule nb demi-tons à ajouter/retrancher
		fi	
		if [ -z $3 ] # teste si un registre à été capturé
			then
				paramRegistre="0" # pas de registre alors valeur 0
			else
				paramRegistre=$(registreVersMidi $3) # calcule nb d'octaves (12 demi-tons) à ajouter/retrancher
		fi	
		echo "scale=2; a = $nbMidi + $paramAlteration + $paramRegistre ; a " | bc		
fi
}

# Cette fonction converti la note midi en fréquence à partir de la table:
# http://www.inspiredacoustics.com/en/MIDI_note_numbers_and_center_frequencies

function noteMidiVersFreq
{
#registre piano midi 21 à midi 108
midiFreqTableau=([0]=0.00 [21]=27.50 [22]=29.14 [23]=30.87 [24]=32.70 [25]=34.65 [26]=36.71 [27]=38.89 [28]=41.20 [29]=43.65 [30]=46.25 [31]=49.00 [32]=51.91 [33]=55.00 [34]=58.27 [35]=61.74 [36]=65.41 [37]=69.30 [38]=73.42 [39]=77.78 [40]=82.41 [41]=87.31 [42]=92.50 [43]=98.00 [44]=103.83 [45]=110.00 [46]=116.54 [47]=123.47 [48]=130.81 [49]=138.59 [50]=146.83 [51]=155.56 [52]=164.81 [53]=174.61 [54]=185.00 [55]=196.00 [56]=207.65 [57]=220 [58]=233.08 [59]=246.94 [60]=261.63 [61]=277.18 [62]=293.66 [63]=311.13 [64]=329.63 [65]=349.23 [66]=369.99 [67]=392.00 [68]=415.30] [69]=440.00 [70]=466.16 [71]=493.88 [72]=523.25 [73]=554.37 [74]=587.33 [75]=622.25 [76]=659.26 [77]=698.46 [78]=739.99 [79]=783.99 [80]=830.61 [81]=880.00 [82]=932.33 [83]=987.77 [84]=1046.50 [85]=1108.73 [86]=1174.66 [87]=1244.51 [88]=1318.51 [89]=1396.91 [90]=1479.98 [91]=1567.98 [92]=1661.22 [93]=1760.00 [94]=1864.66 [95]=1975.53 [96]=2093.00 [97]=2217.46 [98]=2349.32 [99]=2489.02 [100]=2637.02 [101]=2793 [102]29.59.96 [103]=3135.96 [104]=3322.44 [105]=3520.00 [106]=3729.31 [107]=3951.07 [108]=4186.01)
echo ${midiFreqTableau[$1]}
}




# Cette fonction détermine la durée de la note par le nombre entier de la notation lilypond
# elle renvoie le résultat de la division de la mesure par le facteur durée, selon le principe 
# de la description des durées dans les notations musicales anglo-saxonnes et germanique: whole note, half note, quarter note etc.
# pour "1"   durée = mesure
# pour "2"   durée = mesure/2
# pour "n"   durée = mesure/n
# etc.

function dureeVersMs
{
if [  -z $4 ] 	# test si un durée définie explicitement dans l'expression musicale
then  #si pas définie alors
	if [[ -z $2 ]] #teste si la durée précédante est indéfinie ou nulle
		then		
			echo "$3" #si oui, prendre la valeur par défaut.
		else
			echo  "$2"  #si durée précédante est définie s'en servir
		fi
else # si la durée est définie explicitement mesure/durée
		echo "scale=2; a = $1 / $4; a" | bc 
fi			
}




#Cette fonction détermine le facteur de multiplication correspondant aux pointage de la durée

function pointage
{
if [ -z $1 ] 
		then
			echo "1" #note non pointée, renvoie facteur de multiplication 1
		else 
			case $1 in
				".")
					echo "1.5"  #note pointée, renvoie facteur de multiplication 1.5 (1 + 1/2)
					;;
				"..")
					echo "1.75" #note double pointée, renvoie le facteur de multiplication 1.75 (1 + 1/2 + 1/4)
					;;
			esac
fi
}





#-------Le script------
echo "Tempo?:"
read tempo
mesure=$(echo "scale=4; a= 60/$tempo ; a *= 4000; a" | bc)
dureeDefaut=$(echo "scale=4; a= $mesure / 4; a" | bc)
for i in "$@"
do
	if  [[ $1 =~ $regex ]]
	then	
		#calcul de la durée
		dureeBeep=$(echo "scale=2; $(dureeVersMs $mesure $dureeNmoinsUn $dureeDefaut ${BASH_REMATCH[4]}) * $(pointage ${BASH_REMATCH[5]})"| bc) 		
		dureeNmoinsUn=$dureeBeep #défini valeur de dureeNmoinsUn pour la prochaine itération
		# conversion note (altération et registre) en note midi, puis conversion en fréquence				
		frequence=$(noteMidiVersFreq $(hauteurVersMidi ${BASH_REMATCH[1]} ${BASH_REMATCH[2]} ${BASH_REMATCH[3]} ))
	# beep!!!!		
		echo "beep -f "$frequence" -l "$dureeBeep""
		shift	
	else
	echo "Erreur, pas de match regex!!!"		
	fi
done

unset name1
unset name2
unset name3
unset name4
unset name5
unset name6
unset name7
unset rest
unset altup
unset altdown
unset note
unset regex
unset tempo
unset mesure 
unset dureeDefaut
unset dureeBeep
unset dureeNmoinsUn

exit 
